process.env.NODE_ENV = 'test';

const chai = require('chai');

const dirtyChai = require('dirty-chai');

chai.use(dirtyChai);

const chaiHttp = require('chai-http');

const assert = chai.assert;

const expect = chai.expect;

const should = chai.should(); // actually call the function

const server = require('../libs/app');

const mongoose = require('../libs/db/mongoose');
const Simulation = require('../libs/models/simulation');

chai.use(chaiHttp);

describe('request enrollments', () => {
    it('should list enrollments', (done) => {
        chai.request(server)
            .get('/api/v1/enrollment')
            .end((err, res) => {
                res.should.have.status(200);
                res.should.be.json;
                res.body.should.have.property('result');
                res.body.result.should.be.a('array');
                res.body.result[0].should.have.property('name');
                res.body.result[0].should.have.property('total');
                done();
            });
    });
});

describe('request regions', () => {
    it('should list all regions', (done) => {
        chai.request(server)
            .get('/api/v1/region')
            .end((err, res) => {
                res.should.have.status(200);
                res.should.be.json;
                res.body.should.have.property('result');
                res.body.result.should.be.a('array');
                res.body.result[0].should.have.property('pk_regiao_id');
                res.body.result[0].should.have.property('nome');
                done();
            });
    });

    it('should list region by id', (done) => {
        chai.request(server)
            .get('/api/v1/region/1')
            .end((err, res) => {
                res.should.have.status(200);
                res.should.be.json;
                res.body.should.have.property('result');
                res.body.result.should.be.a('array');
                res.body.result.should.have.length(1);
                res.body.result[0].should.have.property('pk_regiao_id');
                res.body.result[0].should.have.property('nome');
                done();
            });
    });
});

describe('request states', () => {
    it('should list all states', (done) => {
        chai.request(server)
            .get('/api/v1/state')
            .end((err, res) => {
                res.should.have.status(200);
                res.should.be.json;
                res.body.should.have.property('result');
                res.body.result.should.be.a('array');
                res.body.result[0].should.have.property('pk_estado_id');
                res.body.result[0].should.have.property('fk_regiao_id');
                res.body.result[0].should.have.property('nome');
                done();
            });
    });

    it('should list a state by id', (done) => {
        chai.request(server)
            .get('/api/v1/state/11')
            .end((err, res) => {
                res.should.have.status(200);
                res.should.be.json;
                res.body.should.have.property('result');
                res.body.result.should.be.a('array');
                res.body.result.should.have.length(1);
                res.body.result[0].should.have.property('pk_estado_id');
                res.body.result[0].should.have.property('fk_regiao_id');
                res.body.result[0].should.have.property('nome');
                done();
            });
    });

    it('should list states by region id', (done) => {
        chai.request(server)
            .get('/api/v1/state/region/1')
            .end((err, res) => {
                res.should.have.status(200);
                res.should.be.json;
                res.body.should.have.property('result');
                res.body.result.should.be.a('array');
                res.body.result[0].should.have.property('pk_estado_id');
                res.body.result[0].should.have.property('fk_regiao_id');
                res.body.result[0].should.have.property('nome');
                done();
            });
    });
});

describe('request cities', () => {
    it('should list all cities', (done) => {
        chai.request(server)
            .get('/api/v1/city')
            .end((err, res) => {
                res.should.have.status(200);
                res.should.be.json;
                res.body.should.have.property('result');
                res.body.result.should.be.a('array');
                res.body.result[0].should.have.property('pk_municipio_id');
                res.body.result[0].should.have.property('fk_estado_id');
                res.body.result[0].should.have.property('nome');
                res.body.result[0].should.have.property('codigo_ibge');
                done();
            });
    });

    it('should list a city by id', (done) => {
        chai.request(server)
            .get('/api/v1/city/1')
            .end((err, res) => {
                res.should.have.status(200);
                res.should.be.json;
                res.body.should.have.property('result');
                res.body.result.should.be.a('array');
                res.body.result[0].should.have.property('pk_municipio_id');
                res.body.result[0].should.have.property('fk_estado_id');
                res.body.result[0].should.have.property('nome');
                res.body.result[0].should.have.property('codigo_ibge');
                done();
            });
    });

    it('should list a city by codigo_ibge', (done) => {
        chai.request(server)
            .get('/api/v1/city/ibge/1200013')
            .end((err, res) => {
                res.should.have.status(200);
                res.should.be.json;
                res.body.should.have.property('result');
                res.body.result.should.be.a('array');
                res.body.result[0].should.have.property('pk_municipio_id');
                res.body.result[0].should.have.property('fk_estado_id');
                res.body.result[0].should.have.property('nome');
                res.body.result[0].should.have.property('codigo_ibge');
                done();
            });
    });
});

describe('Requires a simulation', () => {
    let newSimulation;

    beforeEach(() => {
        Simulation.remove({}, (err) => {
            console.log('Test collection purged');
        });
    });

    it('should create a new simulation', (done) => {
        chai.request(server)
            .post('/api/v1/simulation')
            .set('content-type', 'application/x-www-form-urlencoded')
            .set('x-apicache-bypass', 'true')
            .send({ name: 'test_entry' })
            .end((err, res) => {
                res.should.have.status(200);
                res.should.be.json;
                res.body.should.have.property('id');
                res.body.id.should.be.a('string');
                Simulation.findById(res.body.id, (err, simulation) => {
                    simulation.should.have.property('name');
                    simulation.name.should.be.a('string');
                    simulation.name.should.equal('test_entry');
                });
                done();
            });
    });
    it('should find an existing simulation', (done) => {
        newSimulation = new Simulation();
        newSimulation.name = 'test';
        newSimulation.save((err, sim) => {
            let id = sim._id;
            chai.request(server)
                .get(`/api/v1/simulation/${id}`)
                .end((err, res) => {
                    res.should.have.status(200);
                    res.should.be.json;
                    res.body.should.have.property('_id');
                    res.body._id.should.be.a('string');
                    res.body.should.have.property('name');
                    res.body._id.should.be.a('string');
                    done();
                });
        });
    });
    it('should update an existing simulation\'s location', (done) => {
        newSimulation = new Simulation();
        newSimulation.name = 'test';
        newSimulation.save((err, sim) => {
            let id = sim._id;
            chai.request(server)
                .post(`/api/v1/simulation/${id}`)
                .send({ location: 5 })
                .end((err, res) => {
                    res.should.have.status(200);
                    res.should.be.json;
                    res.body.should.have.property('id');
                    res.body.id.should.be.a('string');
                    Simulation.findById(res.body.id, (err, simulation) => {
                        simulation.should.have.property('name');
                        simulation.name.should.be.a('string');
                        simulation.should.have.property('location');
                        simulation.location.should.be.a('number');
                        simulation.location.should.equal(5);
                    });
                    done();
                });
        });
    });
    it('should update an existing simulation\'s time', (done) => {
        newSimulation = new Simulation();
        newSimulation.name = 'test';
        newSimulation.save((err, sim) => {
            let id = sim._id;
            chai.request(server)
                .post(`/api/v1/simulation/${id}`)
                .send({ time: 5 })
                .end((err, res) => {
                    res.should.have.status(200);
                    res.should.be.json;
                    res.body.should.have.property('id');
                    res.body.id.should.be.a('string');
                    Simulation.findById(res.body.id, (err, simulation) => {
                        simulation.should.have.property('name');
                        simulation.name.should.be.a('string');
                        simulation.should.have.property('time');
                        simulation.time.should.be.a('number');
                        simulation.time.should.equal(5);
                    });
                    done();
                });
        });
    });
});
